#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <GL/gl.h>
#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>
#include "common.h"
#include "fatal.h"

#include "preset_types.h"
#include "preset.h"

#include "glf.h"
#include "glConsole.h"

#include "interface_types.h"

extern interface_t current_interface;

gl_console_t * browser_console = NULL;
int active_font = 0;
char input_buffer[MAX_PATH_SIZE];
int buf_pos = 0;

int loadBrowser() { 

  glfSetCurrentFont(active_font = glfLoadFont("/etc/projectM/fonts/courier1.glf"));

  if ((browser_console = glConsoleCreate(40, 10, 80, 20, 1, 1, active_font)) < 0)
    return FAILURE;

  glConsoleSetFGColor(CONSOLE_RED, browser_console);
  glConsoleSetBGColor(CONSOLE_BLACK, browser_console);
  glConsoleSetFlags(CONSOLE_FLAG_CURSOR_BLINKING | CONSOLE_FLAG_ALWAYS_DRAW_BACKGROUND, browser_console);
  glConsoleClearBuffer(browser_console);

  buf_pos = 0;
  memset(input_buffer, 0, MAX_PATH_SIZE);

  return SUCCESS;
}

int closeBrowser() { 

  glfUnloadFontD(active_font);
  active_font = 0;

  glConsoleDestroy(browser_console);
  browser_console = NULL;

  return SUCCESS;
}

void browser_key_handler(SDL_Event event) {
  char s[2];
  
  s[0] = 0;
  s[1] = 0;
 
    
    switch( event.type ) {
    case SDL_KEYDOWN:	 		    
      switch(event.key.keysym.sym) {
      case SDLK_UP:
	glConsoleMoveCursorUp(browser_console);
	break;
      case SDLK_RETURN:
	loadPresetByFile(input_buffer);
	closeBrowser();
	current_interface = DEFAULT_INTERFACE;
	break;
      case SDLK_RIGHT:
	glConsoleMoveCursorForward(browser_console);
	break;
      case SDLK_LEFT:
	printf("CURSOR BACKWARD\n");
	glConsoleMoveCursorBackward(browser_console);
	break;
      case SDLK_DOWN:
	glConsoleMoveCursorDown(browser_console);
	break;
      case SDLK_PAGEUP:
	glConsoleAlignCursorUp(browser_console);
	break;
      case SDLK_PAGEDOWN:
	glConsoleAlignCursorDown(browser_console);
	break;
      case SDLK_INSERT:
	glConsoleAlignCursorLeft(browser_console);
	break;
      case SDLK_DELETE:
	glConsoleAlignCursorRight(browser_console);
	break;
      case SDLK_LSHIFT:
	break;
      case SDLK_RSHIFT:
	break;
      case SDLK_CAPSLOCK:
	break;
      case SDLK_ESCAPE:
	closeBrowser();
	current_interface = DEFAULT_INTERFACE;
	break;	     
   	
      default: /* All regular characters */
	if (buf_pos == MAX_PATH_SIZE) {
	  buf_pos = 0; 
	}
	input_buffer[buf_pos] = (char)event.key.keysym.sym;
	if ((event.key.keysym.mod == KMOD_LSHIFT) || (event.key.keysym.mod == KMOD_RSHIFT) || (event.key.keysym.mod == KMOD_CAPS))
	  input_buffer[buf_pos] -= 32;

	*s = input_buffer[buf_pos];
	glConsolePrintString(s, browser_console);
	buf_pos++;
	break;
      }
    }
  
  
}

void refreshBrowser() {

  // glConsoleClearBuffer(browser_console);
  //  glConsoleSetCursorPos(1, 1, browser_console);
  //glConsolePrintString("Enter a file to load:\n", browser_console);
  //glConsolePrintString(input_buffer, browser_console);
  glConsoleDraw(browser_console);
}

