#include <stdio.h>
#include <SDL/SDL.h>
#include "common.h"
#include "fatal.h"
#include "menu.h"
#include "interface_types.h"
#include "console_interface.h"
#include "preset_types.h"
#include "preset.h"
#include "engine_vars.h"
#include "browser.h"
#include "editor.h"
#include "video_init.h"


extern preset_t * active_preset;
extern int noSwitch,studio;
extern int fullscreen;
extern int showfps;
extern int showtitle;
extern int showpreset,showhelp;
extern int wvw,wvh,fvw,fvh,vh,vw;

interface_t current_interface = DEFAULT_INTERFACE;

void refreshConsole() {

  switch (current_interface) {

  case MENU_INTERFACE:
    refreshMenu();
    break;
  case SHELL_INTERFACE:
    break;
  case EDITOR_INTERFACE:
    refreshEditor();
    break;
  case DEFAULT_INTERFACE:
    break;
  case BROWSER_INTERFACE:
    refreshBrowser();
    break;
  default:
    break;
  }
 
}


void key_handler() {

  SDL_Event event; 

   while( SDL_PollEvent( &event ) ) {
		  
	switch( event.type ) {

	case SDL_VIDEORESIZE:
	  wvw=event.resize.w;
	  wvh=event.resize.h;
	  vw=wvw;
	  vh=wvh;
	  init_display(vw,vh,0);

	  break;
	case SDL_KEYUP:

	 

	case SDL_KEYDOWN:

	  //default_key_handler();
	  switch (current_interface) 
	    {
	      
	    case MENU_INTERFACE:
	      menu_key_handler(event);
	      break;
	    case SHELL_INTERFACE:
	      //shell_key_handler();
	      break;
	    case EDITOR_INTERFACE:
	      editor_key_handler(event);
	      break;
	    case BROWSER_INTERFACE:
	      browser_key_handler(event);
	      break;
	    case DEFAULT_INTERFACE:
	      default_key_handler(event);
	      break;
	    default:
	      default_key_handler(event);
	      break;
	      
	    }
	  break;
	}
   }
}

void default_key_handler( SDL_Event event ) {
  

  
		  
	switch( event.type ) {

	case SDL_KEYDOWN:
	 		    
	  switch(event.key.keysym.sym)
	    {
	    case SDLK_F1:
	      showhelp++;
	      break;
	    case SDLK_F5:
	      showfps++;
	      break;
	    case SDLK_F3:
	      showpreset++;
	      break;
	    case SDLK_F2:
	      showtitle++;
	      break;
	    case SDLK_F9:
	      studio++;
	      break;
	   
	    case SDLK_f:
	      if (fullscreen==1)
		{vw=wvw;vh=wvh;fullscreen=0;}
	      else{vw=fvw; vh=fvh;fullscreen=1;}

	      init_display(vw,vh,fullscreen);

	      // SDL_WM_ToggleFullScreen(SDL_GetVideoSurface());
	      break;
	    case SDLK_b:
	      break;
            case SDLK_n:
	      if (switchPreset(ALPHA_NEXT, HARD_CUT) < 0) {
		printf("WARNING: Bad preset file, loading idle preset\n");
		switchToIdlePreset();
	      }		
	      break;
	    case SDLK_r:
	      if (switchPreset(RANDOM_NEXT, HARD_CUT) < 0) {
		printf("WARNING: Bad preset file, loading idle preset\n");
		switchToIdlePreset();
	      }	
	      break;
	    case SDLK_p:
	      if ((switchPreset(ALPHA_PREVIOUS, HARD_CUT)) < 0){
		printf("WARNING: Bad preset file, loading idle preset\n");
		switchToIdlePreset();
	      }
	      break;
	    case SDLK_l:
	      if (noSwitch==0)noSwitch=1; else  noSwitch=0;
	      //	      current_interface = BROWSER_INTERFACE;
	      //	      loadBrowser();
	      break;
	    case SDLK_e:      	      	    
	      current_interface = EDITOR_INTERFACE;
	      loadEditor(active_preset->per_frame_eqn_string_buffer, reloadPerFrame, 
			 80, 24, 140, 60, 0, 0);
	      break;
	    case SDLK_s:
	      current_interface = EDITOR_INTERFACE;
	      loadEditor("[FILE NAME HERE]", savePreset, 
			 50, 1, 100, 5, 0, .92);
	    case SDLK_z:      	     	      	     
	      break;
	    case SDLK_0:      
	      nWaveMode=0;
	      break;
	    case SDLK_6:
	      nWaveMode=6;
	      break;
	    case SDLK_7:
	      nWaveMode=7;
	      break;
	    case SDLK_m:
	      current_interface = MENU_INTERFACE;
	      showMenu();
	      break;	     
	    case SDLK_t:
	      break;
	    default:
	      break;
	    }
	}
   



}





